@@ -22,7 +22,6 @@
 #include <config.h>
 #endif
 #include <string.h>
-#include <strings.h>
 #include <gst/video/video.h>
 #include "gsttextoverlay.h"
 
@@ -67,6 +66,8 @@ enum
 #define DEFAULT_XPAD 25
 #define DEFAULT_FONT "sans"
 
+#define GST_CAIRO_TEXT_OVERLAY_DEFAULT_SCALE   20.0
+
 static GstStaticPadTemplate cairo_text_overlay_src_template_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -114,9 +115,10 @@ static void gst_text_overlay_font_init (GstCairoTextOverlay * overlay);
 #define I420_SIZE(w,h)     (I420_V_OFFSET(w,h)+(I420_V_ROWSTRIDE(w)*GST_ROUND_UP_2(h)/2))
 
 GST_BOILERPLATE (GstCairoTextOverlay, gst_text_overlay, GstElement,
-    GST_TYPE_ELEMENT)
+    GST_TYPE_ELEMENT);
 
-     static void gst_text_overlay_base_init (gpointer g_class)
+static void
+gst_text_overlay_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
@@ -247,9 +249,6 @@ gst_text_overlay_init (GstCairoTextOverlay * overlay,
   overlay->need_render = TRUE;
 
   overlay->font = g_strdup (DEFAULT_FONT);
-  overlay->slant = CAIRO_FONT_SLANT_NORMAL;
-  overlay->weight = CAIRO_FONT_WEIGHT_NORMAL;
-  overlay->scale = 20;
   gst_text_overlay_font_init (overlay);
 
   overlay->fps_n = 0;
@@ -273,6 +272,33 @@ gst_text_overlay_font_init (GstCairoTextOverlay * overlay)
   cairo_font_extents_t font_extents;
   cairo_surface_t *surface;
   cairo_t *cr;
+  gchar *font_desc, *sep;
+
+  font_desc = g_ascii_strdown (overlay->font, -1);
+
+  /* cairo_select_font_face() does not parse the size at the end, so we have
+   * to do that ourselves; same for slate and weight */
+  sep = MAX (strrchr (font_desc, ' '), strrchr (font_desc, ','));
+  if (sep != NULL && g_strtod (sep, NULL) > 0.0) {
+    /* there may be a suffix such as 'px', but we just ignore that for now */
+    overlay->scale = g_strtod (sep, NULL);
+  } else {
+    overlay->scale = GST_CAIRO_TEXT_OVERLAY_DEFAULT_SCALE;
+  }
+  if (strstr (font_desc, "bold"))
+    overlay->weight = CAIRO_FONT_WEIGHT_BOLD;
+  else
+    overlay->weight = CAIRO_FONT_WEIGHT_NORMAL;
+
+  if (strstr (font_desc, "italic"))
+    overlay->slant = CAIRO_FONT_SLANT_ITALIC;
+  else if (strstr (font_desc, "oblique"))
+    overlay->slant = CAIRO_FONT_SLANT_OBLIQUE;
+  else
+    overlay->slant = CAIRO_FONT_SLANT_NORMAL;
+
+  GST_LOG_OBJECT (overlay, "Font desc: '%s', scale=%f, weight=%d, slant=%d",
+      overlay->font, overlay->scale, overlay->weight, overlay->slant);
 
   surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 256, 256);
   cr = cairo_create (surface);
@@ -280,12 +306,16 @@ gst_text_overlay_font_init (GstCairoTextOverlay * overlay)
   cairo_select_font_face (cr, overlay->font, overlay->slant, overlay->weight);
   cairo_set_font_size (cr, overlay->scale);
 
+  /* this has a static leak:
+   * http://lists.freedesktop.org/archives/cairo/2007-May/010623.html
+   */
   cairo_font_extents (cr, &font_extents);
   overlay->font_height = GST_ROUND_UP_2 ((guint) font_extents.height);
   overlay->need_render = TRUE;
 
   cairo_destroy (cr);
   cairo_surface_destroy (surface);
+  g_free (font_desc);
 }
 
 static void
@@ -309,11 +339,11 @@ gst_text_overlay_set_property (GObject * object, guint prop_id,
     case ARG_VALIGN:{
       const gchar *s = g_value_get_string (value);
 
-      if (strcasecmp (s, "baseline") == 0)
+      if (g_ascii_strcasecmp (s, "baseline") == 0)
         overlay->valign = GST_CAIRO_TEXT_OVERLAY_VALIGN_BASELINE;
-      else if (strcasecmp (s, "bottom") == 0)
+      else if (g_ascii_strcasecmp (s, "bottom") == 0)
         overlay->valign = GST_CAIRO_TEXT_OVERLAY_VALIGN_BOTTOM;
-      else if (strcasecmp (s, "top") == 0)
+      else if (g_ascii_strcasecmp (s, "top") == 0)
         overlay->valign = GST_CAIRO_TEXT_OVERLAY_VALIGN_TOP;
       else
         g_warning ("Invalid 'valign' property value: %s", s);
@@ -322,11 +352,11 @@ gst_text_overlay_set_property (GObject * object, guint prop_id,
     case ARG_HALIGN:{
       const gchar *s = g_value_get_string (value);
 
-      if (strcasecmp (s, "left") == 0)
+      if (g_ascii_strcasecmp (s, "left") == 0)
         overlay->halign = GST_CAIRO_TEXT_OVERLAY_HALIGN_LEFT;
-      else if (strcasecmp (s, "right") == 0)
+      else if (g_ascii_strcasecmp (s, "right") == 0)
         overlay->halign = GST_CAIRO_TEXT_OVERLAY_HALIGN_RIGHT;
-      else if (strcasecmp (s, "center") == 0)
+      else if (g_ascii_strcasecmp (s, "center") == 0)
         overlay->halign = GST_CAIRO_TEXT_OVERLAY_HALIGN_CENTER;
       else
         g_warning ("Invalid 'halign' property value: %s", s);
@@ -380,17 +410,16 @@ gst_text_overlay_render_text (GstCairoTextOverlay * overlay,
   if (textlen < 0)
     textlen = strlen (text);
 
-  string = g_strndup (text, textlen);
-
-  if (overlay->need_render) {
-    GST_DEBUG ("Rendering text '%s' on cairo RGBA surface", string);
-  } else {
+  if (!overlay->need_render) {
     GST_DEBUG ("Using previously rendered text.");
     g_return_if_fail (overlay->text_fill_image != NULL);
     g_return_if_fail (overlay->text_outline_image != NULL);
     return;
   }
 
+  string = g_strndup (text, textlen);
+  GST_DEBUG ("Rendering text '%s' on cairo RGBA surface", string);
+
   overlay->text_fill_image =
       g_realloc (overlay->text_fill_image,
       4 * overlay->width * overlay->font_height);
