@@ -30,14 +30,18 @@
 GST_DEBUG_CATEGORY_STATIC (rtpmp4apay_debug);
 #define GST_CAT_DEFAULT (rtpmp4apay_debug)
 
-/* FIXME: add framed=(boolean)true once our encoders have this field set
- * on their output caps */
+/* elementfactory information */
+static const GstElementDetails gst_rtp_mp4apay_details =
+GST_ELEMENT_DETAILS ("RTP packet payloader",
+    "Codec/Payloader/Network",
+    "Payload MPEG4 audio as RTP packets (RFC 3016)",
+    "Wim Taymans <wim.taymans@gmail.com>");
+
 static GstStaticPadTemplate gst_rtp_mp4a_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/mpeg, mpegversion=(int)4, "
-        "stream-format=(string)raw")
+    GST_STATIC_CAPS ("audio/mpeg," "mpegversion=(int) 4")
     );
 
 static GstStaticPadTemplate gst_rtp_mp4a_pay_src_template =
@@ -57,42 +61,76 @@ GST_STATIC_PAD_TEMPLATE ("src",
     )
     );
 
+
+static void gst_rtp_mp4a_pay_class_init (GstRtpMP4APayClass * klass);
+static void gst_rtp_mp4a_pay_base_init (GstRtpMP4APayClass * klass);
+static void gst_rtp_mp4a_pay_init (GstRtpMP4APay * rtpmp4apay);
 static void gst_rtp_mp4a_pay_finalize (GObject * object);
 
 static gboolean gst_rtp_mp4a_pay_setcaps (GstBaseRTPPayload * payload,
     GstCaps * caps);
+static GstStateChangeReturn gst_rtp_mp4a_pay_change_state (GstElement * element,
+    GstStateChange transition);
 static GstFlowReturn gst_rtp_mp4a_pay_handle_buffer (GstBaseRTPPayload *
     payload, GstBuffer * buffer);
 
-GST_BOILERPLATE (GstRtpMP4APay, gst_rtp_mp4a_pay, GstBaseRTPPayload,
-    GST_TYPE_BASE_RTP_PAYLOAD)
+static GstBaseRTPPayloadClass *parent_class = NULL;
 
-     static void gst_rtp_mp4a_pay_base_init (gpointer klass)
+static GType
+gst_rtp_mp4a_pay_get_type (void)
+{
+  static GType rtpmp4apay_type = 0;
+
+  if (!rtpmp4apay_type) {
+    static const GTypeInfo rtpmp4apay_info = {
+      sizeof (GstRtpMP4APayClass),
+      (GBaseInitFunc) gst_rtp_mp4a_pay_base_init,
+      NULL,
+      (GClassInitFunc) gst_rtp_mp4a_pay_class_init,
+      NULL,
+      NULL,
+      sizeof (GstRtpMP4APay),
+      0,
+      (GInstanceInitFunc) gst_rtp_mp4a_pay_init,
+    };
+
+    rtpmp4apay_type =
+        g_type_register_static (GST_TYPE_BASE_RTP_PAYLOAD, "GstRtpMP4APay",
+        &rtpmp4apay_info, 0);
+  }
+  return rtpmp4apay_type;
+}
+
+static void
+gst_rtp_mp4a_pay_base_init (GstRtpMP4APayClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_mp4a_pay_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_mp4a_pay_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_mp4a_pay_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_mp4a_pay_sink_template));
 
-  gst_element_class_set_details_simple (element_class,
-      "RTP MPEG4 audio payloader", "Codec/Payloader/Network/RTP",
-      "Payload MPEG4 audio as RTP packets (RFC 3016)",
-      "Wim Taymans <wim.taymans@gmail.com>");
+  gst_element_class_set_details (element_class, &gst_rtp_mp4apay_details);
 }
 
 static void
 gst_rtp_mp4a_pay_class_init (GstRtpMP4APayClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
   GstBaseRTPPayloadClass *gstbasertppayload_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gobject_class->finalize = gst_rtp_mp4a_pay_finalize;
 
+  gstelement_class->change_state = gst_rtp_mp4a_pay_change_state;
+
   gstbasertppayload_class->set_caps = gst_rtp_mp4a_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_mp4a_pay_handle_buffer;
 
@@ -101,7 +139,7 @@ gst_rtp_mp4a_pay_class_init (GstRtpMP4APayClass * klass)
 }
 
 static void
-gst_rtp_mp4a_pay_init (GstRtpMP4APay * rtpmp4apay, GstRtpMP4APayClass * klass)
+gst_rtp_mp4a_pay_init (GstRtpMP4APay * rtpmp4apay)
 {
   rtpmp4apay->rate = 90000;
   rtpmp4apay->profile = g_strdup ("1");
@@ -127,7 +165,7 @@ gst_rtp_mp4a_pay_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static const unsigned int sampling_table[16] = {
+static unsigned sampling_table[16] = {
   96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050,
   16000, 12000, 11025, 8000, 7350, 0, 0, 0
 };
@@ -216,24 +254,21 @@ wrong_channels:
   }
 }
 
-static gboolean
+static void
 gst_rtp_mp4a_pay_new_caps (GstRtpMP4APay * rtpmp4apay)
 {
   gchar *config;
   GValue v = { 0 };
-  gboolean res;
 
   g_value_init (&v, GST_TYPE_BUFFER);
   gst_value_set_buffer (&v, rtpmp4apay->config);
   config = gst_value_serialize (&v);
 
-  res = gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4apay),
+  gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4apay),
       "cpresent", G_TYPE_STRING, "0", "config", G_TYPE_STRING, config, NULL);
 
   g_value_unset (&v);
   g_free (config);
-
-  return res;
 }
 
 static gboolean
@@ -242,28 +277,11 @@ gst_rtp_mp4a_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
   GstRtpMP4APay *rtpmp4apay;
   GstStructure *structure;
   const GValue *codec_data;
-  gboolean res, framed = TRUE;
-  const gchar *stream_format;
 
   rtpmp4apay = GST_RTP_MP4A_PAY (payload);
 
   structure = gst_caps_get_structure (caps, 0);
 
-  /* this is already handled by the template caps, but it is better
-   * to leave here to have meaningful warning messages when linking
-   * fails */
-  stream_format = gst_structure_get_string (structure, "stream-format");
-  if (stream_format) {
-    if (strcmp (stream_format, "raw") != 0) {
-      GST_WARNING_OBJECT (rtpmp4apay, "AAC's stream-format must be 'raw', "
-          "%s is not supported", stream_format);
-      return FALSE;
-    }
-  } else {
-    GST_WARNING_OBJECT (rtpmp4apay, "AAC's stream-format not specified, "
-        "assuming 'raw'");
-  }
-
   codec_data = gst_structure_get_value (structure, "codec_data");
   if (codec_data) {
     GST_LOG_OBJECT (rtpmp4apay, "got codec_data");
@@ -272,6 +290,7 @@ gst_rtp_mp4a_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
       guint8 *config;
       guint8 *data;
       guint size, i;
+      gboolean res;
 
       buffer = gst_value_get_buffer (codec_data);
       GST_LOG_OBJECT (rtpmp4apay, "configuring codec_data");
@@ -317,16 +336,12 @@ gst_rtp_mp4a_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
     }
   }
 
-  if (gst_structure_get_boolean (structure, "framed", &framed) && !framed) {
-    GST_WARNING_OBJECT (payload, "Need framed AAC data as input!");
-  }
-
   gst_basertppayload_set_options (payload, "audio", TRUE, "MP4A-LATM",
       rtpmp4apay->rate);
 
-  res = gst_rtp_mp4a_pay_new_caps (rtpmp4apay);
+  gst_rtp_mp4a_pay_new_caps (rtpmp4apay);
 
-  return res;
+  return TRUE;
 
   /* ERRORS */
 config_failed:
@@ -348,7 +363,6 @@ gst_rtp_mp4a_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   guint count, mtu, size;
   guint8 *data;
   gboolean fragmented;
-  GstClockTime timestamp;
 
   ret = GST_FLOW_OK;
 
@@ -356,7 +370,6 @@ gst_rtp_mp4a_pay_handle_buffer (GstBaseRTPPayload * basepayload,
 
   size = GST_BUFFER_SIZE (buffer);
   data = GST_BUFFER_DATA (buffer);
-  timestamp = GST_BUFFER_TIMESTAMP (buffer);
 
   fragmented = FALSE;
   mtu = GST_BASE_RTP_PAYLOAD_MTU (rtpmp4apay);
@@ -416,22 +429,39 @@ gst_rtp_mp4a_pay_handle_buffer (GstBaseRTPPayload * basepayload,
     /* marker only if the packet is complete */
     gst_rtp_buffer_set_marker (outbuf, size == 0);
 
-    /* copy incomming timestamp (if any) to outgoing buffers */
-    GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
-
     ret = gst_basertppayload_push (GST_BASE_RTP_PAYLOAD (rtpmp4apay), outbuf);
 
     fragmented = TRUE;
   }
-
-  gst_buffer_unref (buffer);
-
   return ret;
 }
 
+static GstStateChangeReturn
+gst_rtp_mp4a_pay_change_state (GstElement * element, GstStateChange transition)
+{
+  GstRtpMP4APay *rtpmp4apay;
+  GstStateChangeReturn ret;
+
+  rtpmp4apay = GST_RTP_MP4A_PAY (element);
+
+  switch (transition) {
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  switch (transition) {
+    default:
+      break;
+  }
+  return ret;
+}
+
+
 gboolean
 gst_rtp_mp4a_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpmp4apay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_MP4A_PAY);
+      GST_RANK_NONE, GST_TYPE_RTP_MP4A_PAY);
 }
