@@ -1,6 +1,7 @@
 /* GStreamer
  * Copyright (C) <2005> Philippe Khalaf <burger@speedy.org>
  * Copyright (C) <2005> Nokia Corporation <kai.vehmanen@nokia.com>
+ * Copyright (C) <2006> Joni Valtanen <joni.valtanen@movial.fi>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,9 +24,21 @@
 #endif
 #include "gstudp-marshal.h"
 #include "gstdynudpsink.h"
+
+#include <stdio.h>
+#include <stdlib.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <errno.h>
+#include <string.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#include <sys/types.h>
 #include <gst/netbuffer/gstnetbuffer.h>
 
-GST_DEBUG_CATEGORY (dynudpsink_debug);
+GST_DEBUG_CATEGORY_STATIC (dynudpsink_debug);
 #define GST_CAT_DEFAULT (dynudpsink_debug)
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -52,13 +65,26 @@ enum
   LAST_SIGNAL
 };
 
+#define UDP_DEFAULT_SOCKFD		-1
+#define UDP_DEFAULT_CLOSEFD		TRUE
+
 enum
 {
   PROP_0,
-  PROP_SOCKFD
-      /* FILL ME */
+  PROP_SOCKFD,
+  PROP_CLOSEFD
 };
 
+#define CLOSE_IF_REQUESTED(udpctx)                                        \
+G_STMT_START {                                                            \
+  if ((!udpctx->externalfd) || (udpctx->externalfd && udpctx->closefd)) { \
+    CLOSE_SOCKET(udpctx->sock);                                           \
+    if (udpctx->sock == udpctx->sockfd)                                   \
+      udpctx->sockfd = UDP_DEFAULT_SOCKFD;                                \
+  }                                                                       \
+  udpctx->sock = -1;                                                      \
+} G_STMT_END
+
 static void gst_dynudpsink_base_init (gpointer g_class);
 static void gst_dynudpsink_class_init (GstDynUDPSink * klass);
 static void gst_dynudpsink_init (GstDynUDPSink * udpsink);
@@ -141,8 +167,12 @@ gst_dynudpsink_class_init (GstDynUDPSink * klass)
 
   g_object_class_install_property (gobject_class, PROP_SOCKFD,
       g_param_spec_int ("sockfd", "socket handle",
-          "Socket to use for UDP reception.",
-          0, G_MAXINT16, 0, G_PARAM_READWRITE));
+          "Socket to use for UDP sending. (-1 == allocate)",
+          -1, G_MAXINT16, UDP_DEFAULT_SOCKFD, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_CLOSEFD,
+      g_param_spec_boolean ("closefd", "Close sockfd",
+          "Close sockfd if passed as property on state change",
+          UDP_DEFAULT_CLOSEFD, G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_dynudpsink_change_state;
 
@@ -151,13 +181,14 @@ gst_dynudpsink_class_init (GstDynUDPSink * klass)
   GST_DEBUG_CATEGORY_INIT (dynudpsink_debug, "dynudpsink", 0, "UDP sink");
 }
 
-
 static void
 gst_dynudpsink_init (GstDynUDPSink * sink)
 {
-  GstDynUDPSink *udpsink;
+  WSA_STARTUP (sink);
 
-  udpsink = GST_DYNUDPSINK (sink);
+  sink->sockfd = UDP_DEFAULT_SOCKFD;
+  sink->closefd = UDP_DEFAULT_CLOSEFD;
+  sink->externalfd = FALSE;
 
   sink->sock = -1;
 }
@@ -165,7 +196,16 @@ gst_dynudpsink_init (GstDynUDPSink * sink)
 static void
 gst_dynudpsink_finalize (GObject * object)
 {
+  GstDynUDPSink *udpsink;
+
+  udpsink = GST_DYNUDPSINK (object);
+
+  if (udpsink->sockfd >= 0 && udpsink->closefd)
+    CLOSE_SOCKET (udpsink->sockfd);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
+
+  WSA_CLEANUP (object);
 }
 
 static GstFlowReturn
@@ -233,8 +273,14 @@ gst_dynudpsink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_SOCKFD:
-      udpsink->sock = g_value_get_int (value);
-      GST_DEBUG ("setting SOCKFD to %d", udpsink->sock);
+      if (udpsink->sockfd >= 0 && udpsink->sockfd != udpsink->sock &&
+          udpsink->closefd)
+        CLOSE_SOCKET (udpsink->sockfd);
+      udpsink->sockfd = g_value_get_int (value);
+      GST_DEBUG ("setting SOCKFD to %d", udpsink->sockfd);
+      break;
+    case PROP_CLOSEFD:
+      udpsink->closefd = g_value_get_boolean (value);
       break;
 
     default:
@@ -253,9 +299,11 @@ gst_dynudpsink_get_property (GObject * object, guint prop_id, GValue * value,
 
   switch (prop_id) {
     case PROP_SOCKFD:
-      g_value_set_int (value, udpsink->sock);
+      g_value_set_int (value, udpsink->sockfd);
+      break;
+    case PROP_CLOSEFD:
+      g_value_set_boolean (value, udpsink->closefd);
       break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -270,10 +318,9 @@ gst_dynudpsink_init_send (GstDynUDPSink * sink)
   guint bc_val;
   gint ret;
 
-  if (sink->sock == -1) {
+  if (sink->sockfd == -1) {
     /* create sender socket if none available */
-
-    if ((sink->sock = socket (AF_INET, SOCK_DGRAM, 0)) == -1)
+    if ((sink->sock = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
       goto no_socket;
 
     bc_val = 1;
@@ -281,8 +328,12 @@ gst_dynudpsink_init_send (GstDynUDPSink * sink)
             setsockopt (sink->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
                 sizeof (bc_val))) < 0)
       goto no_broadcast;
-  }
 
+    sink->externalfd = TRUE;
+  } else {
+    sink->sock = sink->sockfd;
+    sink->externalfd = TRUE;
+  }
   return TRUE;
 
   /* ERRORS */
@@ -294,6 +345,7 @@ no_socket:
 no_broadcast:
   {
     perror ("setsockopt");
+    CLOSE_IF_REQUESTED (sink);
     return FALSE;
   }
 }
@@ -307,7 +359,7 @@ gst_dynudpsink_get_stats (GstDynUDPSink * sink, const gchar * host, gint port)
 static void
 gst_dynudpsink_close (GstDynUDPSink * sink)
 {
-  close (sink->sock);
+  CLOSE_IF_REQUESTED (sink);
 }
 
 static GstStateChangeReturn
