@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2007> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -36,7 +36,15 @@
 #include "gstudp-marshal.h"
 #include "gstmultiudpsink.h"
 
-GST_DEBUG_CATEGORY (multiudpsink_debug);
+#include <stdio.h>
+#include <stdlib.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <errno.h>
+#include <string.h>
+
+GST_DEBUG_CATEGORY_STATIC (multiudpsink_debug);
 #define GST_CAT_DEFAULT (multiudpsink_debug)
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -45,11 +53,11 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS_ANY);
 
 /* elementfactory information */
-static GstElementDetails gst_multiudpsink_details =
+static const GstElementDetails gst_multiudpsink_details =
 GST_ELEMENT_DETAILS ("UDP packet sender",
     "Sink/Network",
     "Send data over the network via UDP",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 /* MultiUDPSink signals and args */
 enum
@@ -68,15 +76,44 @@ enum
   LAST_SIGNAL
 };
 
+#define DEFAULT_SOCKFD             -1
+#define DEFAULT_CLOSEFD            TRUE
+#define DEFAULT_SOCK               -1
+#define DEFAULT_CLIENTS            NULL
+/* FIXME, this should be disabled by default, we don't need to join a multicast
+ * group for sending, if this socket is also used for receiving, it should
+ * be configured in the element that does the receive. */
+#define DEFAULT_AUTO_MULTICAST     TRUE
+#define DEFAULT_TTL                64
+#define DEFAULT_LOOP               TRUE
+#define DEFAULT_QOS_DSCP           -1
+
 enum
 {
   PROP_0,
   PROP_BYTES_TO_SERVE,
   PROP_BYTES_SERVED,
-
-  /* FILL ME */
+  PROP_SOCKFD,
+  PROP_CLOSEFD,
+  PROP_SOCK,
+  PROP_CLIENTS,
+  PROP_AUTO_MULTICAST,
+  PROP_TTL,
+  PROP_LOOP,
+  PROP_QOS_DSCP,
+  PROP_LAST
 };
 
+#define CLOSE_IF_REQUESTED(udpctx)                                        \
+G_STMT_START {                                                            \
+  if ((!udpctx->externalfd) || (udpctx->externalfd && udpctx->closefd)) { \
+    CLOSE_SOCKET(udpctx->sock);                                           \
+    if (udpctx->sock == udpctx->sockfd)                                   \
+      udpctx->sockfd = DEFAULT_SOCKFD;                                    \
+  }                                                                       \
+  udpctx->sock = DEFAULT_SOCK;                                            \
+} G_STMT_END
+
 static void gst_multiudpsink_base_init (gpointer g_class);
 static void gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass);
 static void gst_multiudpsink_init (GstMultiUDPSink * udpsink);
@@ -92,6 +129,13 @@ static void gst_multiudpsink_set_property (GObject * object, guint prop_id,
 static void gst_multiudpsink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
+static void gst_multiudpsink_add_internal (GstMultiUDPSink * sink,
+    const gchar * host, gint port, gboolean lock);
+static void gst_multiudpsink_clear_internal (GstMultiUDPSink * sink,
+    gboolean lock);
+
+static void free_client (GstUDPClient * client);
+
 static GstElementClass *parent_class = NULL;
 
 static guint gst_multiudpsink_signals[LAST_SIGNAL] = { 0 };
@@ -144,7 +188,7 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_BASE_SINK);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->set_property = gst_multiudpsink_set_property;
   gobject_class->get_property = gst_multiudpsink_get_property;
@@ -241,6 +285,39 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
       g_param_spec_uint64 ("bytes-served", "Bytes served",
           "Total number of bytes send to all clients", 0, G_MAXUINT64, 0,
           G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class, PROP_SOCKFD,
+      g_param_spec_int ("sockfd", "Socket Handle",
+          "Socket to use for UDP sending. (-1 == allocate)",
+          -1, G_MAXINT, DEFAULT_SOCKFD, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_CLOSEFD,
+      g_param_spec_boolean ("closefd", "Close sockfd",
+          "Close sockfd if passed as property on state change",
+          DEFAULT_CLOSEFD, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_SOCK,
+      g_param_spec_int ("sock", "Socket Handle",
+          "Socket currently in use for UDP sending. (-1 == no socket)",
+          -1, G_MAXINT, DEFAULT_SOCK, G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class, PROP_CLIENTS,
+      g_param_spec_string ("clients", "Clients",
+          "A comma separated list of host:port pairs with destinations",
+          DEFAULT_CLIENTS, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_AUTO_MULTICAST,
+      g_param_spec_boolean ("auto-multicast",
+          "Automatically join/leave multicast groups",
+          "Automatically join/leave the multicast groups, FALSE means user"
+          " has to do it himself", DEFAULT_AUTO_MULTICAST, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_TTL,
+      g_param_spec_int ("ttl", "Multicast TTL",
+          "Used for setting the multicast TTL parameter",
+          0, 255, DEFAULT_TTL, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_LOOP,
+      g_param_spec_boolean ("loop", "Multicast Loopback",
+          "Used for setting the multicast loop parameter. TRUE = enable,"
+          " FALSE = disable", DEFAULT_LOOP, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_QOS_DSCP,
+      g_param_spec_int ("qos-dscp", "QoS diff srv code point",
+          "Quality of Service, differentiated services code point (-1 default)",
+          -1, 63, DEFAULT_QOS_DSCP, G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_multiudpsink_change_state;
 
@@ -257,7 +334,17 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
 static void
 gst_multiudpsink_init (GstMultiUDPSink * sink)
 {
+  WSA_STARTUP (sink);
+
   sink->client_lock = g_mutex_new ();
+  sink->sock = DEFAULT_SOCK;
+  sink->sockfd = DEFAULT_SOCKFD;
+  sink->closefd = DEFAULT_CLOSEFD;
+  sink->externalfd = (sink->sockfd != -1);
+  sink->auto_multicast = DEFAULT_AUTO_MULTICAST;
+  sink->ttl = DEFAULT_TTL;
+  sink->loop = DEFAULT_LOOP;
+  sink->qos_dscp = DEFAULT_QOS_DSCP;
 }
 
 static void
@@ -267,8 +354,16 @@ gst_multiudpsink_finalize (GObject * object)
 
   sink = GST_MULTIUDPSINK (object);
 
+  g_list_foreach (sink->clients, (GFunc) free_client, NULL);
+  g_list_free (sink->clients);
+
+  if (sink->sockfd >= 0 && sink->closefd)
+    CLOSE_SOCKET (sink->sockfd);
+
   g_mutex_free (sink->client_lock);
 
+  WSA_CLEANUP (object);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -287,7 +382,7 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
 
   sink->bytes_to_serve += size;
 
-  /* grab lock while iterating and sending to clients, this shuld be
+  /* grab lock while iterating and sending to clients, this should be
    * fast as UDP never blocks */
   g_mutex_lock (sink->client_lock);
   GST_LOG_OBJECT (bsink, "about to send %d bytes", size);
@@ -304,6 +399,10 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
           (struct sockaddr *) &client->theiraddr, sizeof (client->theiraddr));
 
       if (ret < 0) {
+        /* we get a non-posix EPERM on Linux when a firewall rule blocks this
+         * destination. We will simply ignore this. */
+        if (errno == EPERM)
+          break;
         if (errno != EINTR && errno != EAGAIN) {
           goto send_error;
         }
@@ -333,6 +432,89 @@ send_error:
   }
 }
 
+static void
+gst_multiudpsink_set_clients_string (GstMultiUDPSink * sink,
+    const gchar * string)
+{
+  gchar **clients;
+  gint i;
+
+  clients = g_strsplit (string, ",", 0);
+
+  g_mutex_lock (sink->client_lock);
+  /* clear all existing clients */
+  gst_multiudpsink_clear_internal (sink, FALSE);
+  for (i = 0; clients[i]; i++) {
+    gchar *host, *p;
+    gint port = 0;
+
+    host = clients[i];
+    p = strstr (clients[i], ":");
+    if (p != NULL) {
+      *p = '\0';
+      port = atoi (p + 1);
+    }
+    if (port != 0)
+      gst_multiudpsink_add_internal (sink, host, port, FALSE);
+  }
+  g_mutex_unlock (sink->client_lock);
+
+  g_strfreev (clients);
+}
+
+static gchar *
+gst_multiudpsink_get_clients_string (GstMultiUDPSink * sink)
+{
+  GString *str;
+  GList *clients;
+
+  str = g_string_new ("");
+
+  g_mutex_lock (sink->client_lock);
+  clients = sink->clients;
+  while (clients) {
+    GstUDPClient *client;
+
+    client = (GstUDPClient *) clients->data;
+
+    clients = g_list_next (clients);
+
+    g_string_append_printf (str, "%s:%d%s", client->host, client->port,
+        (clients ? "," : ""));
+  }
+  g_mutex_unlock (sink->client_lock);
+
+  return g_string_free (str, FALSE);
+}
+
+static void
+gst_multiudpsink_setup_qos_dscp (GstMultiUDPSink * sink)
+{
+  gint tos;
+
+  /* don't touch on -1 */
+  if (sink->qos_dscp < 0)
+    return;
+
+  if (sink->sock < 0)
+    return;
+
+  GST_DEBUG_OBJECT (sink, "setting TOS to %d", sink->qos_dscp);
+
+  /* Extract and shift 6 bits of DSFIELD */
+  tos = (sink->qos_dscp & 0x3f) << 2;
+
+  if (setsockopt (sink->sock, IPPROTO_IP, IP_TOS, &tos, sizeof (tos)) < 0) {
+    GST_ERROR_OBJECT (sink, "could not set TOS: %s", g_strerror (errno));
+  }
+#ifdef IPV6_TCLASS
+  if (setsockopt (sink->sock, IPPROTO_IPV6, IPV6_TCLASS, &tos,
+          sizeof (tos)) < 0) {
+    GST_ERROR_OBJECT (sink, "could not set TCLASS: %s", g_strerror (errno));
+  }
+#endif
+}
+
 static void
 gst_multiudpsink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -342,6 +524,32 @@ gst_multiudpsink_set_property (GObject * object, guint prop_id,
   udpsink = GST_MULTIUDPSINK (object);
 
   switch (prop_id) {
+    case PROP_SOCKFD:
+      if (udpsink->sockfd >= 0 && udpsink->sockfd != udpsink->sock &&
+          udpsink->closefd)
+        CLOSE_SOCKET (udpsink->sockfd);
+      udpsink->sockfd = g_value_get_int (value);
+      GST_DEBUG_OBJECT (udpsink, "setting SOCKFD to %d", udpsink->sockfd);
+      break;
+    case PROP_CLOSEFD:
+      udpsink->closefd = g_value_get_boolean (value);
+      break;
+    case PROP_CLIENTS:
+      gst_multiudpsink_set_clients_string (udpsink, g_value_get_string (value));
+      break;
+    case PROP_AUTO_MULTICAST:
+      udpsink->auto_multicast = g_value_get_boolean (value);
+      break;
+    case PROP_TTL:
+      udpsink->ttl = g_value_get_int (value);
+      break;
+    case PROP_LOOP:
+      udpsink->loop = g_value_get_boolean (value);
+      break;
+    case PROP_QOS_DSCP:
+      udpsink->qos_dscp = g_value_get_int (value);
+      gst_multiudpsink_setup_qos_dscp (udpsink);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -363,24 +571,58 @@ gst_multiudpsink_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_BYTES_SERVED:
       g_value_set_uint64 (value, udpsink->bytes_served);
       break;
-
+    case PROP_SOCKFD:
+      g_value_set_int (value, udpsink->sockfd);
+      break;
+    case PROP_CLOSEFD:
+      g_value_set_boolean (value, udpsink->closefd);
+      break;
+    case PROP_SOCK:
+      g_value_set_int (value, udpsink->sock);
+      break;
+    case PROP_CLIENTS:
+      g_value_take_string (value,
+          gst_multiudpsink_get_clients_string (udpsink));
+      break;
+    case PROP_AUTO_MULTICAST:
+      g_value_set_boolean (value, udpsink->auto_multicast);
+      break;
+    case PROP_TTL:
+      g_value_set_int (value, udpsink->ttl);
+      break;
+    case PROP_LOOP:
+      g_value_set_boolean (value, udpsink->loop);
+      break;
+    case PROP_QOS_DSCP:
+      g_value_set_int (value, udpsink->qos_dscp);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-
 /* create a socket for sending to remote machine */
 static gboolean
 gst_multiudpsink_init_send (GstMultiUDPSink * sink)
 {
   guint bc_val;
   gint ret;
+  GList *clients;
+  GstUDPClient *client;
 
-  /* create sender socket */
-  if ((sink->sock = socket (AF_INET, SOCK_DGRAM, 0)) == -1)
-    goto no_socket;
+  if (sink->sockfd == -1) {
+    /* create sender socket try IP6, fall back to IP4 */
+    if ((sink->sock = socket (AF_INET6, SOCK_DGRAM, 0)) == -1)
+      if ((sink->sock = socket (AF_INET, SOCK_DGRAM, 0)) == -1)
+        goto no_socket;
+
+    sink->externalfd = FALSE;
+  } else {
+    /* we use the configured socket */
+    sink->sock = sink->sockfd;
+    sink->externalfd = TRUE;
+  }
 
   bc_val = 1;
   if ((ret =
@@ -391,6 +633,15 @@ gst_multiudpsink_init_send (GstMultiUDPSink * sink)
   sink->bytes_to_serve = 0;
   sink->bytes_served = 0;
 
+  gst_udp_set_loop_ttl (sink->sock, sink->loop, sink->ttl);
+  gst_multiudpsink_setup_qos_dscp (sink);
+
+  /* look for multicast clients and join multicast groups appropriately */
+  for (clients = sink->clients; clients; clients = g_list_next (clients)) {
+    client = (GstUDPClient *) clients->data;
+    if (sink->auto_multicast && gst_udp_is_multicast (&client->theiraddr))
+      gst_udp_join_group (*(client->sock), &client->theiraddr);
+  }
   return TRUE;
 
   /* ERRORS */
@@ -402,6 +653,7 @@ no_socket:
   }
 no_broadcast:
   {
+    CLOSE_IF_REQUESTED (sink);
     GST_ELEMENT_ERROR (sink, RESOURCE, SETTINGS, (NULL),
         ("Could not set broadcast socket option (%d): %s", errno,
             g_strerror (errno)));
@@ -412,15 +664,13 @@ no_broadcast:
 static void
 gst_multiudpsink_close (GstMultiUDPSink * sink)
 {
-  close (sink->sock);
+  CLOSE_IF_REQUESTED (sink);
 }
 
-void
-gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
+static void
+gst_multiudpsink_add_internal (GstMultiUDPSink * sink, const gchar * host,
+    gint port, gboolean lock)
 {
-  struct hostent *he;
-  struct in_addr addr;
-  struct ip_mreq multi_addr;
   GstUDPClient *client;
   GTimeVal now;
 
@@ -430,62 +680,55 @@ gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
   client->port = port;
   client->sock = &sink->sock;
 
-  memset (&client->theiraddr, 0, sizeof (client->theiraddr));
-  client->theiraddr.sin_family = AF_INET;       /* host byte order */
-  client->theiraddr.sin_port = htons (port);    /* short, network byte order */
+  if (gst_udp_get_addr (host, port, &client->theiraddr) < 0)
+    goto getaddrinfo_error;
 
   g_get_current_time (&now);
   client->connect_time = GST_TIMEVAL_TO_TIME (now);
 
-  /* if its an IP address */
-  if (inet_aton (host, &addr)) {
+  if (*client->sock > 0) {
     /* check if its a multicast address */
-    if ((ntohl (addr.s_addr) & 0xe0000000) == 0xe0000000) {
-      client->multi_addr.imr_multiaddr.s_addr = addr.s_addr;
-      client->multi_addr.imr_interface.s_addr = INADDR_ANY;
-
-      client->theiraddr.sin_addr = multi_addr.imr_multiaddr;
-
-      /* Joining the multicast group */
-      /* FIXME, can we use multicast and unicast over the same
-       * socket? if not, search for socket of this multicast group or
-       * create a new one. */
-      setsockopt (sink->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, &multi_addr,
-          sizeof (multi_addr));
+    if (gst_udp_is_multicast (&client->theiraddr)) {
+      GST_DEBUG_OBJECT (sink, "multicast address detected");
+      if (sink->auto_multicast) {
+        GST_DEBUG_OBJECT (sink, "joining multicast group");
+        gst_udp_join_group (*(client->sock), &client->theiraddr);
+      }
     } else {
-      client->theiraddr.sin_addr = *((struct in_addr *) &addr);
+      GST_DEBUG_OBJECT (sink, "normal address detected");
     }
   }
-  /* we dont need to lookup for localhost */
-  else if (strcmp (host, "localhost") == 0 && inet_aton ("127.0.0.1", &addr)) {
-    client->theiraddr.sin_addr = *((struct in_addr *) &addr);
-  }
-  /* if its a hostname */
-  else if ((he = gethostbyname (host))) {
-    client->theiraddr.sin_addr = *((struct in_addr *) he->h_addr);
-  } else {
-    goto host_error;
-  }
 
-  g_mutex_lock (sink->client_lock);
+  if (lock)
+    g_mutex_lock (sink->client_lock);
   sink->clients = g_list_prepend (sink->clients, client);
-  g_mutex_unlock (sink->client_lock);
+  if (lock)
+    g_mutex_unlock (sink->client_lock);
 
   g_signal_emit (G_OBJECT (sink),
       gst_multiudpsink_signals[SIGNAL_CLIENT_ADDED], 0, host, port);
 
+  GST_DEBUG_OBJECT (sink, "added client on host %s, port %d", host, port);
   return;
 
   /* ERRORS */
-host_error:
+getaddrinfo_error:
   {
-    GST_WARNING_OBJECT (sink, "hostname lookup error?");
+    GST_DEBUG_OBJECT (sink, "did not add client on host %s, port %d", host,
+        port);
+    GST_WARNING_OBJECT (sink, "getaddrinfo lookup error?");
     g_free (client->host);
     g_free (client);
     return;
   }
 }
 
+void
+gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
+{
+  gst_multiudpsink_add_internal (sink, host, port, TRUE);
+}
+
 static gint
 client_compare (GstUDPClient * a, GstUDPClient * b)
 {
@@ -526,6 +769,10 @@ gst_multiudpsink_remove (GstMultiUDPSink * sink, const gchar * host, gint port)
   g_get_current_time (&now);
   client->disconnect_time = GST_TIMEVAL_TO_TIME (now);
 
+  if (*(client->sock) != -1 && sink->auto_multicast
+      && gst_udp_is_multicast (&client->theiraddr))
+    gst_udp_leave_group (*(client->sock), &client->theiraddr);
+
   /* Unlock to emit signal before we delete the actual client */
   g_mutex_unlock (sink->client_lock);
   g_signal_emit (G_OBJECT (sink),
@@ -549,17 +796,25 @@ not_found:
   }
 }
 
-void
-gst_multiudpsink_clear (GstMultiUDPSink * sink)
+static void
+gst_multiudpsink_clear_internal (GstMultiUDPSink * sink, gboolean lock)
 {
   GST_DEBUG_OBJECT (sink, "clearing");
   /* we only need to remove the client structure, there is no additional
    * socket or anything to free for UDP */
-  g_mutex_lock (sink->client_lock);
+  if (lock)
+    g_mutex_lock (sink->client_lock);
   g_list_foreach (sink->clients, (GFunc) free_client, sink);
   g_list_free (sink->clients);
   sink->clients = NULL;
-  g_mutex_unlock (sink->client_lock);
+  if (lock)
+    g_mutex_unlock (sink->client_lock);
+}
+
+void
+gst_multiudpsink_clear (GstMultiUDPSink * sink)
+{
+  gst_multiudpsink_clear_internal (sink, TRUE);
 }
 
 GValueArray *
